#!/usr/bin/env python3
# depends (debian) python3-lxml
import sys
import email
from base64 import b64encode
from lxml import html as HTML

def iter(e):
    yield e
    for _e in e.iterchildren():
        yield from iter(_e)

def drop_banner(h):
    for e in iter(h):
        if any((x in e.attrib.get('id', []) for x in ('cropswap-banner', 'cropswap-members-banner'))):
            e.drop_tree()
    return HTML.tostring(h).decode("utf-8")

mailgroup = sys.argv[1]
try:
    with open(f'/opt/cropswap/{mailgroup}/banner.html', 'rb') as f:
        html = f.read().decode("utf-8")
    with open(f'/opt/cropswap/{mailgroup}/banner.txt', 'rb') as f:
        txt = f.read().decode("utf-8")
except FileNotFoundError:
    html = ''
    txt = ''

m = email.message_from_bytes(sys.stdin.buffer.read())

INJECTED = {
    'text/plain': False,
    'text/html': False,
}

def recode(msg, payload, encoder):
    _bytes = payload.encode("utf-8")
    if encoder is email.encoders.encode_base64:
        msg.set_payload(b64encode(_bytes))
    else:
        msg.set_payload(_bytes)
    encoder(msg)

def inject(m):
    c = m.get_content_type()
    e = m.get('Content-Transfer-Encoding','').lower()
    payload = m.get_payload(decode=True).decode(m.get_content_charset("utf-8"))

    if c == 'text/plain' and txt not in payload:
        payload += f"\n{txt}\n"

    elif c == 'text/html':
        h = HTML.fromstring(payload)
        payload = drop_banner(h)

        if '</body>' in payload:
            payload = payload.replace('</body>', f'{html}</body>')
        else:
            payload += f'{html}'

    if e == 'quoted-printable':
        encode = email.encoders.encode_quopri
    elif e in ('7bit', '8bit'):
        encode = email.encoders.encode_7or8bit
    elif e == 'base64':
        encode = email.encoders.encode_base64
    else:
        encode = email.encoders.encode_noop

    recode(m, payload, encode)
    m.set_charset("utf-8")

def visit(m):
    c = m.get_content_type()
    if c in ('text/plain', 'text/html'):
        if INJECTED[c]: return
        inject(m)
        INJECTED[c] = True

for _m in m.walk():
     visit(_m)

sys.stdout.buffer.write(m.as_bytes())
